package cz.vitskalicky.lepsirozvrh

import android.content.Context
import com.fasterxml.jackson.core.JsonProcessingException
import cz.vitskalicky.lepsirozvrh.MainApplication.Companion.objectMapper
import cz.vitskalicky.lepsirozvrh.bakaAPI.rozvrh.rozvrh3.Atom3
import cz.vitskalicky.lepsirozvrh.bakaAPI.rozvrh.rozvrh3.Day3
import cz.vitskalicky.lepsirozvrh.bakaAPI.rozvrh.rozvrh3.Rozvrh3
import cz.vitskalicky.lepsirozvrh.bakaAPI.rozvrh.rozvrh3.RozvrhConverter.convert
import cz.vitskalicky.lepsirozvrh.model.rozvrh.Rozvrh
import org.joda.time.LocalDate
import org.joda.time.LocalTime
import org.joda.time.format.DateTimeFormat
import java.util.*

class DebugUtils constructor(ctx: Context) {
    private val context: MainApplication = ctx.applicationContext as MainApplication

    //demonstration mode
    private val demoRozvrh: Rozvrh? = null
    var isDemoMode: Boolean
        get() = SharedPrefs.getBoolean(context, "demo-mode") && BuildConfig.DEBUG
        set(newDemoMode) {
            SharedPrefs.setBoolean(context, "demo-mode", newDemoMode)
        }



    companion object {
        private val TAG = DebugUtils::class.java.simpleName
        private const val demoRozvrhSource = """{"classes":[{"abbrev":"V5.","id":"ZX","name":"kvinta"}],"cycles":[{"abbrev":"B","id":"1","name":"týden B"}],"days":[{"atoms":[{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"3","roomId":"30","subjectId":"04","teacherId":"UJBNX","theme":"Geometrická interpretace absolutní hodnoty"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"4","roomId":"30","subjectId":" 5","teacherId":"U6Y7G","theme":"L5 - Reflexive Verben"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":["ZXKCMFTK"],"hourId":"5","roomId":"32","subjectId":" 4","teacherId":"UPBOA","theme":"U7 Revision"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"6","roomId":"G4","subjectId":"0E","teacherId":"UTBOJ","theme":"Bílkoviny, sacharidy"},{"change":{"changeSubject":null,"changeType":"Added","day":"2020-09-14T00:00:00+02:00","description":"Spojeno: MA, č. 3 (F)","hours":"5. hod","time":"11:50 - 12:35","typeAbbrev":null,"typeName":null},"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"7","roomId":"30","subjectId":"04","teacherId":"UJBNX","theme":"Vlastnosti absolutní hodnoty"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"8","roomId":"30","subjectId":"04","teacherId":"UJBNX","theme":"Vlastnosti absolutní hodnoty"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"9","roomId":"G0","subjectId":"0D","teacherId":"UUBOK","theme":"Hvězdy"}],"date":"2020-09-14T00:00:00+02:00","dayDescription":"","dayOfWeek":1,"dayType":"WorkDay"},{"atoms":[{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"4","roomId":"C0","subjectId":"12","teacherId":"U3KK3","theme":"lipidy"},{"change":{"changeSubject":null,"changeType":"Substitution","day":"2020-09-15T00:00:00+02:00","description":"Změna hodiny: Ch (F)","hours":"3. hod","time":"10:00 - 10:55","typeAbbrev":null,"typeName":null},"cycleIds":["1"],"groupIds":["WC"],"homeworkIds":["ZXWCGHOB"],"hourId":"5","roomId":"3Z","subjectId":"12","teacherId":"U3KK3","theme":"Typy chem.reakcí"},{"change":null,"cycleIds":["1"],"groupIds":["WC"],"homeworkIds":[],"hourId":"6","roomId":"3Z","subjectId":"12","teacherId":"U3KK3","theme":"Typy chem.reakcí"},{"change":null,"cycleIds":["1"],"groupIds":["WC"],"homeworkIds":[],"hourId":"7","roomId":"LG","subjectId":"0F","teacherId":"UJ8XX","theme":"hardware"},{"change":null,"cycleIds":["1"],"groupIds":["WC"],"homeworkIds":[],"hourId":"8","roomId":"LG","subjectId":"0F","teacherId":"UJ8XX","theme":"hardware-prezentace"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"9","roomId":"30","subjectId":" 4","teacherId":"UPBOA","theme":"U7 Revision, U7 Listening"}],"date":"2020-09-15T00:00:00+02:00","dayDescription":"","dayOfWeek":2,"dayType":"WorkDay"},{"atoms":[{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"3","roomId":"3Z","subjectId":"12","teacherId":"U3KK3","theme":"metabolismus"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"4","roomId":"30","subjectId":" 2","teacherId":"UZBN3","theme":"Literární druhy a žánry"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"5","roomId":"30","subjectId":"04","teacherId":"UJBNX","theme":"Usměrňování zlomků"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"6","roomId":"SR","subjectId":" 5","teacherId":"U6Y7G","theme":"L5 - Wie fühlt ihr euch?"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"7","roomId":"30","subjectId":"0C","teacherId":"UPBO9","theme":"Psychologické školy."},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"8","roomId":"30","subjectId":" 8","teacherId":"UZBN3","theme":"Periodizace pravěku"},{"change":null,"cycleIds":["1"],"groupIds":["WE"],"homeworkIds":[],"hourId":"9","roomId":"VF","subjectId":"0I","teacherId":"UG78B","theme":"Dokončení a hodnocení min. práce"},{"change":null,"cycleIds":["1"],"groupIds":["WE"],"homeworkIds":[],"hourId":"10","roomId":"VF","subjectId":"0I","teacherId":"UG78B","theme":"Pravěké umění"}],"date":"2020-09-16T00:00:00+02:00","dayDescription":"","dayOfWeek":3,"dayType":"WorkDay"},{"atoms":[{"change":{"changeSubject":null,"changeType":"Removed","day":"2020-09-17T00:00:00+02:00","description":"Vyjmuto z rozvrhu (V5, G)","hours":"1. hod","time":"8:00 - 8:55","typeAbbrev":null,"typeName":null},"cycleIds":[],"groupIds":[],"homeworkIds":[],"hourId":"3","roomId":null,"subjectId":null,"teacherId":null,"theme":null},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"4","roomId":"3Z","subjectId":"12","teacherId":"U3KK3","theme":"bílkoviny"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"5","roomId":"DT","subjectId":"0D","teacherId":"UUBOK","theme":"Sluneční soustava"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"6","roomId":"0X","subjectId":" 4","teacherId":"UPBOA","theme":"U8 Imagination - Vocabulary"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"7","roomId":"30","subjectId":" 2","teacherId":"UZBN3","theme":"Složitá souvětí - opakování"},{"change":{"changeSubject":null,"changeType":"Substitution","day":"2020-09-17T00:00:00+02:00","description":"Změna hodiny: ČJL, č. 3 (F, F)","hours":"6. hod","time":"12:45 - 13:30","typeAbbrev":null,"typeName":null},"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"8","roomId":"30","subjectId":" 2","teacherId":"UZBN3","theme":"Pravopis - opakování"},{"change":null,"cycleIds":["1"],"groupIds":["K7"],"homeworkIds":[],"hourId":"9","roomId":"VG","subjectId":"17","teacherId":"UTBOG","theme":"Atletika, skok daleký, nácvik odrazu"}],"date":"2020-09-17T00:00:00+02:00","dayDescription":"","dayOfWeek":4,"dayType":"WorkDay"},{"atoms":[{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"3","roomId":"30","subjectId":"04","teacherId":"UJBNX","theme":"Úpravy výrazů"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"4","roomId":"0X","subjectId":" 5","teacherId":"U6Y7G","theme":"L4 - Wortschatz"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"5","roomId":"30","subjectId":"0C","teacherId":"UPBO9","theme":"Předmět psychologie."},{"change":null,"cycleIds":["1"],"groupIds":["K7"],"homeworkIds":[],"hourId":"6","roomId":"VG","subjectId":"17","teacherId":"UTBOG","theme":"Atletika, skok daleký, letová fáze"},{"change":null,"cycleIds":["1"],"groupIds":["KC"],"homeworkIds":[],"hourId":"7","roomId":"32","subjectId":" 4","teacherId":"UPBOA","theme":"SB The Hobbit - Reading"},{"change":null,"cycleIds":["1"],"groupIds":["K6"],"homeworkIds":[],"hourId":"8","roomId":"30","subjectId":" 2","teacherId":"UZBN3","theme":"Lidová slovesnost"}],"date":"2020-09-18T00:00:00+02:00","dayDescription":"","dayOfWeek":5,"dayType":"WorkDay"}],"groups":[{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. IKT2","classId":"ZX","id":"WC","name":"kvinta IKT2"},{"abbrev":"V5. IKT2","classId":"ZX","id":"WC","name":"kvinta IKT2"},{"abbrev":"V5. IKT2","classId":"ZX","id":"WC","name":"kvinta IKT2"},{"abbrev":"V5. IKT2","classId":"ZX","id":"WC","name":"kvinta IKT2"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. VýO","classId":"ZX","id":"WE","name":"kvinta VýO"},{"abbrev":"V5. VýO","classId":"ZX","id":"WE","name":"kvinta VýO"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. Hoši","classId":"ZX","id":"K7","name":"kvinta hoši"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"},{"abbrev":"V5. Hoši","classId":"ZX","id":"K7","name":"kvinta hoši"},{"abbrev":"V5. 1","classId":"ZX","id":"KC","name":"kvinta 1"},{"abbrev":"V5.","classId":"ZX","id":"K6","name":"kvinta"}],"hours":[{"beginTime":"7:05","caption":"0","endTime":"7:50","id":2},{"beginTime":"8:00","caption":"1","endTime":"8:45","id":3},{"beginTime":"8:55","caption":"2","endTime":"9:40","id":4},{"beginTime":"10:00","caption":"3","endTime":"10:45","id":5},{"beginTime":"10:55","caption":"4","endTime":"11:40","id":6},{"beginTime":"11:50","caption":"5","endTime":"12:35","id":7},{"beginTime":"12:45","caption":"6","endTime":"13:30","id":8},{"beginTime":"14:00","caption":"7","endTime":"14:45","id":9},{"beginTime":"14:45","caption":"8","endTime":"15:30","id":10},{"beginTime":"15:35","caption":"9","endTime":"16:20","id":11},{"beginTime":"16:20","caption":"10","endTime":"17:05","id":12},{"beginTime":"17:20","caption":"11","endTime":"18:05","id":13},{"beginTime":"18:15","caption":"12","endTime":"19:00","id":14}],"rooms":[{"abbrev":"č. 3","id":"30","name":""},{"abbrev":"č.16","id":"32","name":""},{"abbrev":"č.19","id":"G4","name":""},{"abbrev":"č.11","id":"G0","name":""},{"abbrev":"F","id":"C0","name":""},{"abbrev":"č.23","id":"3Z","name":""},{"abbrev":"IKT","id":"LG","name":""},{"abbrev":"č.17","id":"SR","name":""},{"abbrev":"č.20","id":"VF","name":""},{"abbrev":"č.10","id":"DT","name":""},{"abbrev":"č. 4","id":"0X","name":""},{"abbrev":"VT","id":"VG","name":""}],"subjects":[{"abbrev":"MA","id":"04","name":"Matematika a její aplikace"},{"abbrev":"NJ","id":" 5","name":"Německý jazyk"},{"abbrev":"AJ","id":" 4","name":"Anglický jazyk"},{"abbrev":"BiG","id":"0E","name":"Biologie a geologie"},{"abbrev":"G","id":"0D","name":"Geografie"},{"abbrev":"Ch","id":"12","name":"Chemie"},{"abbrev":"IKT","id":"0F","name":"Informační a komunikační technologie"},{"abbrev":"ČJ","id":" 2","name":"Český jazyk a literatura"},{"abbrev":"ZSV","id":"0C","name":"Základy společenských věd"},{"abbrev":"D","id":" 8","name":"Dějepis"},{"abbrev":"Vý","id":"0I","name":"Výtvarný obor"},{"abbrev":"TV","id":"17","name":"Tělesná výchova"}],"teachers":[{"abbrev":"Ko","id":"UJBNX","name":"Mgr. Milan Kohout"},{"abbrev":"Va","id":"U6Y7G","name":"Mgr. Eva Vaculíková"},{"abbrev":"Ho","id":"UPBOA","name":"Matěj Horáček"},{"abbrev":"Ga","id":"UTBOJ","name":"Mgr. Gabriela Gabčová"},{"abbrev":"Hn","id":"UUBOK","name":"Miroslav Hnilička"},{"abbrev":"Ve","id":"U3KK3","name":"RNDr. Vladimír Veselý Ph. D."},{"abbrev":"Be","id":"UJ8XX","name":"Mgr. Jiří Beneš"},{"abbrev":"Ba","id":"UZBN3","name":"Mgr. Renata Bartošová"},{"abbrev":"Kr","id":"UPBO9","name":"Mgr. Amalie Krousová"},{"abbrev":"Bí","id":"UG78B","name":"Miluše Bílá"},{"abbrev":"Ka","id":"UTBOG","name":"Mgr. Alexandra Kahounová"}]}"""

        val demoTime: LocalTime
            get() = LocalTime.parse("9:00")
        val demoDate: LocalDate
            get() = Utils.getCurrentMonday().plusDays(1)

        fun getDemoRozvrh3(mondayDate: LocalDate): Rozvrh3 {
            val dtf = DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ssZZ")
            try {
                val r3 = objectMapper.readValue(demoRozvrhSource, Rozvrh3::class.java)
                val newlist: MutableList<Day3> = ArrayList()
                for (i in r3.days.indices) {
                    val item = r3.days[i]
                    val newDate = mondayDate.plusDays(i)
                    val newAtoms: List<Atom3> = item.atoms.map { it.copy(change = it.change?.copy(day = dtf.print(newDate.toDateTimeAtStartOfDay()))) }
                    newlist.add(item.copy(atoms = newAtoms, date = dtf.print(newDate.toDateTimeAtStartOfDay())))
                }
                return r3.copy(days = newlist)
            } catch (e: JsonProcessingException) {
                throw IllegalStateException("Failed to parse demo rozvrh", e)
            }
        }

        fun getDemoRozvrh(mondayDate: LocalDate, context: Context): Rozvrh {
            return convert(getDemoRozvrh3(mondayDate), demoDate, context)
        }
    }

}